/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch das
 * Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.formcycle;

import static java.util.Objects.*;
import static org.apache.commons.lang3.StringUtils.*;

import java.util.Optional;
import java.util.Properties;

import de.ozgcloud.formcycle.errorhandling.TechnicalException;
import de.xima.fc.interfaces.plugin.lifecycle.IPluginInitializeData;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class OzgPluginPropertiesSupplier {

	private final IPluginInitializeData pluginData;
	private final PortParser portParser;

	public String getEingangsAdapterUrl() {
		return pluginData.getProperties().getProperty(OzgCloudConfig.KEY_EINGANGSADAPTER_URL);
	}

	public Optional<ProxyConfig> getProxyProperties() {
		var pluginProperties = pluginData.getProperties();
		validateProxyConfig(pluginProperties);
		var proxyHostProperty = pluginProperties.getProperty(ProxyConfig.KEY_PROXY_HOST);
		return isBlank(proxyHostProperty) ? Optional.empty() : Optional.of(buildProxyConfig(pluginProperties));
	}

	void validateProxyConfig(Properties pluginProperties) {
		var proxyHostProperty = pluginProperties.getProperty(ProxyConfig.KEY_PROXY_HOST);
		var port = getPort(pluginProperties);
		if ((isBlank(proxyHostProperty) && nonNull(port))
				|| (isNotBlank(proxyHostProperty) && isNull(port))) {
			throw new TechnicalException("Proxy configuration is invalid. Both host and port must be set or none of them.");
		}
	}

	ProxyConfig buildProxyConfig(Properties pluginProperties) {
		return ProxyConfig.builder()
				.host(pluginProperties.getProperty(ProxyConfig.KEY_PROXY_HOST))
				.port(getPort(pluginProperties))
				.user(pluginProperties.getProperty(ProxyConfig.KEY_PROXY_USER))
				.password(pluginProperties.getProperty(ProxyConfig.KEY_PROXY_PASSWORD))
				.build();
	}

	Integer getPort(Properties pluginProperties) {
		var portProperty = pluginProperties.getProperty(ProxyConfig.KEY_PROXY_PORT);
		return isBlank(portProperty) ? null : portParser.parse(portProperty);
	}

}
