package de.ozgcloud.formcycle.errorhandling;

import static de.ozgcloud.formcycle.errorhandling.OzgPluginError.*;
import static java.util.Objects.*;

import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.formcycle.OzgCloudConfig;
import de.ozgcloud.formcycle.OzgCloudPlugin;
import de.xima.fc.exceptions.NodeThrewException;
import de.xima.fc.interfaces.workflow.params.INodeThrewExceptionBuilder;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class NodeThrewExceptionFactory {

	public static final String KEY_EXCEPTION_ID = "exceptionId";

	private final INodeThrewExceptionBuilder exceptionBuilder;
	private final String pluginVersion;
	private final OzgCloudConfig pluginConfig;

	public NodeThrewException createInternalException(TechnicalException e) {
		return exceptionBuilder
				.error(INTERNAL_ERROR.name(), createDataMap(e.getExceptionId()))
				.message(formatWithPluginConfig(e.getMessage()))
				.cause(e).build();
	}

	public NodeThrewException createInternalException(String message, Exception e) {
		return exceptionBuilder
				.error(INTERNAL_ERROR.name(), createDataMap(ExceptionUtil.createExceptionId()))
				.message(formatWithPluginConfig(ExceptionUtil.formatMessageWithExceptionId(message)))
				.cause(e).build();
	}

	public NodeThrewException createOzgCloudRequestException(OzgCloudRequestException e) {
		return exceptionBuilder.error(OZGCLOUD_REQUEST_FAILED.name(), createDataMap(e.getExceptionId()))
				.message(formatWithPluginConfig(e.getMessage()))
				.cause(e).build();
	}

	Map<String, String> createDataMap(String exceptionId) {
		return Map.of(KEY_EXCEPTION_ID, Optional.ofNullable(exceptionId).orElseGet(ExceptionUtil::createExceptionId));
	}

	String formatWithPluginConfig(String message) {
		var builder = StringUtils.isBlank(message) ? new StringBuilder("An error happened.") : new StringBuilder(message);
		builder.append("\nPlugin configuration:")
				.append("\n\t").append(OzgCloudPlugin.KEY_PLUGIN_VERSION).append(": ").append(pluginVersion);
		if (nonNull(pluginConfig)) {
			builder.append("\n\t").append(OzgCloudConfig.KEY_EINGANGSADAPTER_URL).append(": ").append(pluginConfig.getEingangsAdapterUrl());
			if (nonNull(pluginConfig.getProxyConfig())) {
				builder.append("\n\t").append(pluginConfig.getProxyConfig());
			}
		}
		return builder.toString();
	}
}
