package de.ozgcloud.formcycle.formdata;

import static de.ozgcloud.formcycle.formdata.ItemClass.*;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import de.xima.fc.form.common.items.XItem;
import de.xima.fc.form.common.models.XPropertyValue;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
class FormStructureParser {

	static final String CONTAINER_NAME_ANTRAGSTELLER = "fsBKAllDaten";

	private final Map<String, XItem> formularElementsSorted;
	private final Deque<FormNode> containerNodes = new ArrayDeque<>();
	private final List<FormNode> resultNodes = new ArrayList<>();

	public List<FormNode> parse() {
		resultNodes.clear();
		for (Map.Entry<String, XItem> entry : formularElementsSorted.entrySet()) {
			var xItem = entry.getValue();
			var itemClass = ItemClass.fromString(xItem.getClassName());
			if (isInputNode(itemClass)) {
				handleInputNode(xItem);
			} else if (isNestedContainer(itemClass)) {
				handleContainerNode(xItem);
			}
		}
		return resultNodes;
	}

	void handleInputNode(XItem xItem) {
		getResultNodes(xItem).add(buildNode(xItem));
	}

	void handleContainerNode(XItem xItem) {
		var formNode = buildNode(xItem);
		getResultNodes(xItem).add(formNode);
		containerNodes.push(formNode);
	}

	FormNode buildNode(XItem xItem) {
		var formNode = FormNode.builder().itemId(xItem.getId()).name(xItem.getName());
		getTitle(xItem).ifPresent(formNode::title);
		return formNode.build();
	}

	Optional<String> getTitle(XItem xItem) {
		return ItemClass.fromString(xItem.getClassName()).getTitleProperty().map(xItem::get).map(XPropertyValue::getString);
	}

	List<FormNode> getResultNodes(XItem xItem) {
		if (CONTAINER_NAME_ANTRAGSTELLER.equalsIgnoreCase(xItem.getName())) {
			return resultNodes;
		}
		return getNextContainerNode(xItem.getParentId()).map(FormNode::getNestedElements).orElse(resultNodes);
	}

	Optional<FormNode> getNextContainerNode(String parentId) {
		var nextContainerNode = containerNodes.peekFirst();
		if (Objects.isNull(nextContainerNode)) {
			return Optional.empty();
		}
		while (nonParent(nextContainerNode, parentId)) {
			containerNodes.pollFirst();
			if (containerNodes.isEmpty()) {
				return Optional.empty();
			}
			nextContainerNode = containerNodes.peek();
		}
		return Optional.of(nextContainerNode);
	}

	private boolean nonParent(FormNode containerNode, String parentId) {
		return !containerNode.getItemId().equals(parentId);
	}
}
