/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.formcycle.formdata;

import static de.ozgcloud.formcycle.formdata.ItemClass.*;
import static de.ozgcloud.formcycle.formdata.StructureMockFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.Map;

import org.junit.jupiter.api.Test;

import de.xima.fc.form.common.items.XItem;

class FormStructureParserITCase {

	private static final String FS_CONTAINER_ID = "xi-fs-n";
	private static final String NEXT_ITEM_ID = "new-item";

	private static final FormNode EXPECTED_FIELD_SET_NODE = FormNodeTestFactory.createBuilder().values(null).itemId(FS_CONTAINER_ID).build();
	private static final FormNode EXPECTED_TEXT_FIELD_NODE = FormNodeTestFactory.createBuilder().values(null).itemId(NEXT_ITEM_ID).build();

	@Test
	void shouldParseInputItem() {
		var expected = FormNodeTestFactory.createBuilder().values(null).build();

		var formNodes = new FormStructureParser(createItem(TEXT_FIELD)).parse();

		assertThat(formNodes).hasSize(1).first().usingRecursiveComparison().isEqualTo(expected);
	}

	@Test
	void shouldParseContainerItem() {
		var expected = FormNodeTestFactory.createBuilder().values(null).build();

		var formNodes = new FormStructureParser(createItem(FIELD_SET)).parse();

		assertThat(formNodes).hasSize(1).first().usingRecursiveComparison().isEqualTo(expected);
	}

	@Test
	void shouldNotParseSpanItem() {
		var formNodes = new FormStructureParser(createItem(SPAN)).parse();

		assertThat(formNodes).isEmpty();
	}

	@Test
	void shouldParseNestedFieldSetContainer() {
		var formNodes = new FormStructureParser(initNestedMap()).parse();

		assertThat(formNodes).hasSize(2);
		var containerNodes = formNodes.get(0).getNestedElements();
		assertThat(containerNodes).hasSize(2);
		assertThat(containerNodes.get(0)).usingRecursiveComparison().isEqualTo(EXPECTED_FIELD_SET_NODE);
		assertThat(containerNodes.get(1)).usingRecursiveComparison().isEqualTo(EXPECTED_TEXT_FIELD_NODE);
		assertThat(formNodes.get(1).getItemId()).isEqualTo(ITEM_ID);
	}

	private Map<String, XItem> initNestedMap() {
		var containerWithItem = createPageItem();
		containerWithItem.put(CONTAINER_ITEM_ID, getMock(CONTAINER, PAGE_ITEM_ID));
		containerWithItem.put(FS_CONTAINER_ID, getMock(FIELD_SET, FS_CONTAINER_ID, CONTAINER_ITEM_ID, ITEM_TITLE, ITEM_NAME));
		containerWithItem.put(NEXT_ITEM_ID, getMock(TEXT_FIELD, NEXT_ITEM_ID, CONTAINER_ITEM_ID, ITEM_TITLE, ITEM_NAME));
		containerWithItem.put(ITEM_ID, getMock(CHECKBOX, PAGE_ITEM_ID));

		return containerWithItem;
	}
}
